require 'swagger/api/version'
require 'rails'

module Swagger
  module Api
    def self.prettify
      JSON.pretty_generate(JSON.parse(json))
    end

    def self.json
      create.to_json
    end

    def self.create
      @config ||= {
        openapi: '3.0.0',
        security: [{api_key: []}],
        info: info,
        servers: [{url: server_url}],
        paths: Paths.new(controllers: config.controllers).create,
        components: {
          responses: responses,
          schemas: Components.new(controllers: config.controllers).create,
          requestBodies: RequestBodies.new(controllers: config.controllers).create,
          securitySchemes: security_schemes,
        },
      }
    end

    def self.config
      @yaml_config ||= JSON.parse(YAML.load_file("#{Rails.root.to_s}/config/swagger.yml").to_json, object_class: OpenStruct)
    end

    def self.security_schemes
      {
        api_key: {
          type: 'apiKey',
          name: 'Authorization',
          in: 'header'
        }
      }
    end

    def self.responses
      {
        NotFound: {
          description: "The specified resource was not found",
          content: {
            'application/json; charset=utf-8' => {
              schema: {
                type: 'string',
                example: 'Not Found'
              }
            }
          }
        },
        Unauthorized: {
          description: "Unauthorized",
          content: {
            'application/json; charset=utf-8' => {
              schema: {
                type: 'string',
                example: 'Not Authorized'
              }
            }
          }
        },
        BadRequest: {
          description: "Bad Request",
          content: {
            'application/json; charset=utf-8' => {
              schema: {
                example: ['The field name is invalid.', 'The id must be present'],
                type: 'array',
                items: {
                  type: 'string'
                }
              }
            }
          }
        },
        Unexpected: {
          description: "Unexpected Error",
          content: {
            'application/json; charset=utf-8' => {
              schema: {
                type: 'string',
                example: 'Unexpected Error'
              }
            }
          }
        }
      }
    end

    def self.info
      {
        version: config.info.version,
        title: config.info.title,
        description: config.info.description,
      }
    end

    def self.server_url
      config.servers.send(Rails.env).url
    end
  end
end
require 'swagger/railtie'

require 'swagger/api/actions'
require 'swagger/api/concerns/columns'
require 'swagger/api/column_schema'
require 'swagger/api/component_schema'
require 'swagger/api/components'
require 'swagger/api/operations/base'
require 'swagger/api/operations/create'
require 'swagger/api/operations/delete'
require 'swagger/api/operations/index'
require 'swagger/api/operations/show'
require 'swagger/api/operations/update'
require 'swagger/api/paths'
require 'swagger/api/request_bodies'
